/*
 * Modified from board-htcherald.c
 *
 * HTC Artemis init stuff
 * Copyright (C) 2009 Cory Maccarrone <darkstar6262@gmail.com>
 * Copyright (C) 2008 wing-linux.sourceforge.net
 *
 * This  program is  free  software; you  can  redistribute it  and/or
 * modify  it under the  terms of  the GNU  General Public  License as
 * published by the Free Software  Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT  ANY  WARRANTY;  without   even  the  implied  warranty  of
 * MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU
 * General Public License for more details.
 * 
 * You should have  received a copy of the  GNU General Public License
 * along  with  this program;  if  not,  write  to the  Free  Software
 * Foundation,  Inc.,  51 Franklin  Street,  Fifth  Floor, Boston,  MA
 * 02110-1301, USA. 
 *
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/input.h>
#include <linux/bootmem.h>

#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/arch/omap850.h>
#include <asm/page.h>
#include <asm/memory.h>
#include <asm/arch/common.h>
#include <asm/arch/board.h>

#ifdef CONFIG_EFB_DEBUG
#include <asm/arch/efb.h>
#endif

#include <asm/arch/io.h>
#include <asm/arch/irqs.h>
#include <asm/arch/gpio.h>
#include <asm/arch/keypad.h>
#include <asm/arch/spi100k.h>

#include <linux/spi/spi.h>
#include <linux/spi/tsc2046.h>

#include <linux/delay.h>

#ifdef CONFIG_HTC_MMC
#include "htc-mmc.h"
#endif

#define HTCARTEMIS_GPIO_DM 35
#define HTCARTEMIS_GPIO_DP 36
#define HTCARTEMIS_GPIO_POWER 139
#define HTCARTEMIS_GPIO_SLIDE 174

static struct omap_lcd_config htcartemis_lcd_config __initdata = {
	.ctrl_name	= "internal",
};

static struct omap_usb_config htcartemis_usb_config __initdata = {
	.otg		= 0,
	.register_host	= 0,
	.register_dev	= 1,
	.hmc_mode	= 4,
	.pins[0]	= 2,
};

static struct omap_mmc_config htcartemis_mmc_config __initdata =
{
	.mmc[0] = {
		.enabled = 1,
		.nomux = 1,
		.wire4 = 1,
		.power_pin = -1,
		.switch_pin = -1,
	}
};

/*
  UART1 Bluetooth
  UART2 IR?
*/
static struct omap_uart_config htcartemis_uart_config __initdata = {
        .enabled_uarts = (1 << 0) | (1 << 1),
};

static struct omap_board_config_kernel htcartemis_config[] __initdata = 
{
	{ OMAP_TAG_LCD, &htcartemis_lcd_config },
	{ OMAP_TAG_USB, &htcartemis_usb_config },
	{ OMAP_TAG_MMC, &htcartemis_mmc_config },
	{ OMAP_TAG_UART, &htcartemis_uart_config },
};

/* GSM device */
#define ARTEMIS_GSM_PHYS_START          0x13c00000 /* end of RAM */
#define ARTEMIS_GSM_PHYS_SIZE           0x00400000 /* 4.0 MB code/data/fifo */

static struct resource artemis_gsm_resources[] = {
	{       /* GSM DSP MMU */
                .start          = IO_ADDRESS(OMAP850_DSP_MMU_BASE),
                .end            = IO_ADDRESS(OMAP850_DSP_MMU_BASE) + 0x54,
                .flags          = IORESOURCE_MEM,
        },
        {       /* GSM software interrupt */
                .start          = INT_850_ICR,
                .flags          = IORESOURCE_IRQ,
        },
        {       /* GSM radio interrupt */
                .start          = INT_850_DBB_RF_EN,
                .flags          = IORESOURCE_IRQ,
        },
};

static struct platform_device gsm_device = {
        .name           = "wizard-gsm",
        .id             = 1,
        .num_resources  = ARRAY_SIZE(artemis_gsm_resources),
        .resource       = artemis_gsm_resources,
};

/* Keyboard definition */

static int htc_artemis_keymap[] = {
	KEY(2,3,KEY_RECORD), /* Mail button */
	KEY(2,0,KEY_CAMERA), /* Camera */
	KEY(0,0,KEY_PHONE), /* Send key */
	KEY(0,3,KEY_VOLUMEUP), /* Volume up */
	KEY(1,3,KEY_VOLUMEDOWN), /* Volume down */
	KEY(2,2,KEY_MAIL), /* Win key (portrait) */
	KEY(1,2,KEY_DIRECTORY), /* Right bar (protrait) */
	KEY(1,4,KEY_SLASH), /* OK key */
	KEY(0,1,KEY_CONNECT), /* Voice button */
	KEY(1,0,KEY_CANCEL), /* End key */
	KEY(1,1,KEY_WWW), /* OK button (portrait) */
	KEY(0,2,KEY_CALENDAR), /* Left bar (portrait) */
	0
};

struct omap_kp_platform_data htcartemis_kp_data = {
	.rows	= 7,
	.cols	= 7,
	.delay = 20,
	.rep = 1,
	.keymap = htc_artemis_keymap,
};

static struct resource kp_resources[] = {
	[0] = {
		.start	= INT_850_MPUIO_KEYPAD,
		.end	= INT_850_MPUIO_KEYPAD,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device kp_device = {
	.name		= "omap-keypad",
	.id		= -1,
	.dev		= {
		.platform_data = &htcartemis_kp_data,
	},
	.num_resources	= ARRAY_SIZE(kp_resources),
	.resource	= kp_resources,
};

/* GPIO buttons for keyboard slide and power button */
#if defined(CONFIG_KEYBOARD_GPIO) || defined(CONFIG_KEYBOARD_GPIO_MODULE)
#include <linux/input.h>
#include <linux/gpio_keys.h>

static struct gpio_keys_button artemis_gpio_keys_table[] = {
   {BTN_0, HTCARTEMIS_GPIO_POWER, 1, "gpio-keys: POWER", EV_KEY, 1},
   {SW_LID, HTCARTEMIS_GPIO_SLIDE, 0, "gpio-keys: SLIDE", EV_SW, 1, 1},
};
 
static struct gpio_keys_platform_data artemis_gpio_keys_data = {
   .buttons        = artemis_gpio_keys_table,
   .nbuttons       = ARRAY_SIZE(artemis_gpio_keys_table),
};

static struct platform_device artemis_gpiokeys_device = {
   .name      = "gpio-keys",
   .dev = {
      .platform_data = &artemis_gpio_keys_data,
   },
};
#endif

/* LCD Device resources */
static struct platform_device lcd_device = {
	.name           = "lcd_htcherald",
	.id             = -1,
};

/* Bluetooth */
struct platform_device htcartemis_bt = {
   .name = "htc-bt",
   .id = 1,
};

static struct platform_device *devices[] __initdata = {
 	&gsm_device,
	&kp_device,
	&lcd_device,
#if defined(CONFIG_KEYBOARD_GPIO) || defined(CONFIG_KEYBOARD_GPIO_MODULE)
   &artemis_gpiokeys_device,
#endif
#if defined(CONFIG_HTC_BT)
	&htcartemis_bt,
#endif
};

/*
 * Touchscreen
 */
static struct tsc2046_platform_data htcartemis_ts_platform_data __initdata = {
        .ts_x_plate_ohm    = 496, 
        .dav_gpio          = 76,
        .gpio_debounce     = 0,
		  .ts_max_pressure   = 100000,
        .ts_touch_pressure = 5000, 
};

static struct spi_board_info htcartemis_spi_board_info[] __initdata = { 
	{
 	       .modalias               = "tsc2046",
 	       .platform_data          = &htcartemis_ts_platform_data,
	       .max_speed_hz           = 2500000,
	       .bus_num                = 2,
	       .chip_select            = 1,
	       .irq		       = OMAP_GPIO_IRQ(76),
	} 
};

/*
 * Init functions from here on
 */
static void __init htcartemis_map_io(void)
{
	omap1_map_common_io();
	printk("htcartemis_map_io done.\n");

#ifdef CONFIG_EFB_DEBUG
	efb_enable();
#endif

	/* Reserve GSM Memory */
	reserve_bootmem(ARTEMIS_GSM_PHYS_START, ARTEMIS_GSM_PHYS_SIZE, 0);
}

static void __init htcartemis_disable_watchdog(void)
{
  /* Disable watchdog if running */
  if (omap_readl(OMAP_WDT_TIMER_MODE) & 0x8000) {
    /*
     * disable a potentially running watchdog timer before
     * it kills us.
     */
    printk("OMAP850 Watchdog seems to be activated, disabling it for now.\n");
    omap_writel(0xF5, OMAP_WDT_TIMER_MODE);
    omap_writel(0xA0, OMAP_WDT_TIMER_MODE);
  }
}

/* TSC2046 init from board-nokia770.c */


static void __init htcartemis_usb_enable(void)
{
	unsigned int tries = 20;
	printk("trying to enable USB.\n");

	/* force USB_EN GPIO to 0 */
	do {
		omap_set_gpio_direction(33, 0); /* output */
		omap_set_gpio_dataout(33, 0); /* low */
		--tries;
	} while(omap_get_gpio_datain(33) && tries);
	if (tries) {
		printk("unable to reset USB_EN GPIO after 20 tries.\n");
		printk("I will try to continue anyway: USB may not be available.\n");
	}
	printk("USB_EN to 0 after %i tries.\n", tries);

	omap_set_gpio_dataout(73, 0);
	
	omap_set_gpio_direction(HTCARTEMIS_GPIO_DM, 1); /* input */
	
	/* get uart control from GSM */
	
	/* select GPIO35 for D_MCLK_OUT */
	/* select GPIO36 for D_CRESET */
	omap_writel(omap_readl(OMAP730_IO_CONF_3) & 0xffffffcc, OMAP730_IO_CONF_3);
	omap_writel(omap_readl(OMAP730_IO_CONF_3) | 0x000000cc, OMAP730_IO_CONF_3);
	

	omap_set_gpio_direction(HTCARTEMIS_GPIO_DP, 1); /* input */

	/* select D_DM, D_DP for D_DM and disable PE_DM control */
	omap_writel(omap_readl(OMAP730_IO_CONF_2) & 0xff1fffff, OMAP730_IO_CONF_2);
	omap_writel(omap_readl(OMAP730_IO_CONF_2) | 0x00100000, OMAP730_IO_CONF_2);
	mdelay(100);

	/* select USB_VBUSI for D_VBUSI, enable PE_VIBUSI pull enable control  */
	omap_writel(omap_readl(OMAP730_IO_CONF_2) & 0xf1ffffff, OMAP730_IO_CONF_2);
	omap_writel(omap_readl(OMAP730_IO_CONF_2) | 0x01000000, OMAP730_IO_CONF_2);

	/* set USB_VBUS_CTRL */
	omap_writel(omap_readl(OMAP730_MODE_1) | (1 << 25), OMAP730_MODE_1);
}

static void __init htcartemis_usb_otg(void)
{
	/* clock configuration */
	omap_writew(omap_readw(ULPD_SOFT_REQ) | (1 << 8) | SOFT_USB_CLK_REQ, ULPD_SOFT_REQ);

	//  clk_enable(&l3_ocpi_ck);
	omap_writew(omap_readw(ARM_IDLECT3) | (1 << 0), ARM_IDLECT3);

	/* pin muxing */
	omap_writel(omap_readl(OMAP730_MODE_1) & ~(1 <<  2), OMAP730_MODE_1);
	omap_writel(omap_readl(OMAP730_MODE_1) & ~(1 <<  3), OMAP730_MODE_1);
	omap_writel(omap_readl(OMAP730_MODE_1) |  (1 << 15), OMAP730_MODE_1);
	omap_writel(omap_readl(OMAP730_MODE_1) |  (1 << 23), OMAP730_MODE_1);
	omap_writel(omap_readl(OMAP730_MODE_1) |  (1 << 26), OMAP730_MODE_1);
	omap_writel(omap_readl(OMAP730_MODE_1) |  (1 << 25), OMAP730_MODE_1);
	omap_writel(omap_readl(OMAP730_MODE_1) & ~(1 << 24), OMAP730_MODE_1);
	omap_writel(omap_readl(OMAP730_MODE_1) & ~(1 << 10), OMAP730_MODE_1);
	omap_writel(omap_readl(OMAP730_MODE_1) & ~(1 << 11), OMAP730_MODE_1);
}

static void __init htcartemis_spi_mux(void)
{
	/* Setup MUX config for SPI */
	omap_writel(omap_readl(OMAP850_IO_CONF_6) |  0x00088880, OMAP850_IO_CONF_6);
	omap_writel(omap_readl(OMAP850_IO_CONF_6) & ~0x00077770, OMAP850_IO_CONF_6);

	omap_writel(omap_readl(OMAP850_IO_CONF_8) |  0x01000000, OMAP850_IO_CONF_8);
	omap_writel(omap_readl(OMAP850_IO_CONF_8) & ~0x10110000, OMAP850_IO_CONF_8);

	omap_writel(omap_readl(OMAP850_IO_CONF_9) |  0x00000010, OMAP850_IO_CONF_9);
	omap_writel(omap_readl(OMAP850_IO_CONF_9) & ~0x00000001, OMAP850_IO_CONF_9);

	/* configure spi setup registers */
	omap_writew(0xfffe, OMAP850_SPI2_BASE + 0x02);
	omap_writew(0x0000, OMAP850_SPI2_BASE + 0x08);
	omap_writew(0x7ff8, OMAP850_SPI2_BASE + 0x0e);
}

static void __init htcartemis_init(void)
{
  printk("HTC Artemis init.\n");
  efb_putstr("HTC init");
  omap_board_config = htcartemis_config;
  omap_board_config_size = ARRAY_SIZE(htcartemis_config);
  platform_add_devices(devices, ARRAY_SIZE(devices));

  htcartemis_disable_watchdog();

  htcartemis_usb_otg();
  htcartemis_usb_enable();
  htcartemis_spi_mux();
  htcartemis_i2c_init();
#ifdef CONFIG_HTC_MMC
  htc_mmc_init();
#endif

  // Add UART2 clock activation
  omap_writel(omap_readl(0xfffe0834) | 0x800, 0xfffe0834);
  // Add UART1 clock activation
  omap_writel(omap_readl(0xfffe0834) | 0x200, 0xfffe0834);
  omap_serial_init();

  /* For testing.. Disable for now */
  spi_register_board_info(htcartemis_spi_board_info,
			ARRAY_SIZE(htcartemis_spi_board_info));
  /* omap_free_gpio(76); */
}

static void __init htcartemis_init_irq(void)
{
	printk("htcartemis_init_irq.\n");
	omap1_init_common_hw();
	omap_init_irq();
	omap_gpio_init();
}

MACHINE_START(ARTEMIS, "HTC Artemis")
        /* Maintainer: Cory Maccarrone <darkstar6262@gmail.com> */
		  /* Maintainer: wing-linux.sourceforge.net */
        .phys_io        = 0xfff00000,
        .io_pg_offst    = ((0xfef00000) >> 18) & 0xfffc,
        .boot_params    = 0x10000100,
        .map_io         = htcartemis_map_io,
        .init_irq       = htcartemis_init_irq,
        .init_machine   = htcartemis_init,
        .timer          = &omap_timer,
MACHINE_END
