/* Phone interface driver for GPS SIRF-III
 *
 * 2007-12-12 Robert Jarzmik
 *
 * This code is licenced under the GPLv2.
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/device.h>
#include <linux/platform_device.h>

#ifdef CONFIG_I2C_HTCPLD
#include <linux/i2c/i2c-htcpld.h>
#endif

#include <asm/gpio.h>
#include <asm/arch/omap850.h>
#include <asm/mach-types.h>

#include "htc-bt.h"

static int status=0; /* Default bt off */
static int pm=1; /* Default to power management */

static struct htc_bt_driver btdrv;

void htc_bt_init_gpios()
{
   // Configure gpios
   /* configure bt UART clock*/
   //omap_writel(omap_readl(0xfffe0834) | 0x200, 0xfffe0834);
   //

	/* configure UART PIN MUX */
	/* Note: UART / serial pins are shared with GPIO 40,41,42,43
	 * That means doing anything with those GPIOs will do nothing
	 */
	omap_writel((omap_readl(OMAP850_IO_CONF_3) & 0xf11fffff), OMAP850_IO_CONF_3);

	/* configure GPIO 125 (BT power) MUX */
	omap_writel((omap_readl(OMAP850_IO_CONF_8) & 0xfffffff1) | 0x0000000c, OMAP850_IO_CONF_8);

	/* GPIO 125 to output */
	omap_set_gpio_direction(125, 0);

	/* turn it off - functions as a reset too! */
	gpio_set_value(125, 0);

   /* Disable the I2C bit to toggle bluetooth on -- HTC Herald, HTC Elf, and others needs this */
   if  (machine_is_herald() ||
        machine_is_htcelf() ||
        machine_is_htcstartrek() ||		// Not sure about this yet, just copying off of Elf
        machine_is_htcpharos() ||
        machine_is_artemis())
   {
#ifdef CONFIG_I2C_HTCPLD
      htcpld_chip_set(0x04, htcpld_chip_get(0x04) & 0xef);
#endif
   }

   status = 0;
}


#define EGPIO_5_0_BT 22

void htc_bt_on()
{
   /* configure SIRF.III GPIOS */
   gpio_set_value(125, 1);

	/* Is this really necessary? */
   mdelay(1000);

   /* configure bt UART clock*/
   omap_writel(omap_readl(0xfffe0834) | 0x200, 0xfffe0834);

   /* Enable the I2C bit to toggle bluetooth on */
   if  (machine_is_herald() ||
        machine_is_htcelf() ||
        machine_is_htcstartrek() ||		// Not sure about this yet, just copying off of Elf
        machine_is_htcpharos() ||
        machine_is_artemis())
   {
#ifdef CONFIG_I2C_HTCPLD
      htcpld_chip_set(0x04, htcpld_chip_get(0x04) | 0x10);
#endif
   }

   status = 1;

   printk("HTC Bluetooth: Bluetooth turned on.\n");
}

void htc_bt_off()
{
   /* configure bt UART clock*/
   omap_writel(omap_readl(0xfffe0834) & 0xfffffdff, 0xfffe0834);

   /* configure SIRF.III GPIOS */
   gpio_set_value(125, 0);

   /* Disable the I2C bit to toggle bluetooth on */
   if  (machine_is_herald() ||
        machine_is_htcelf() ||
        machine_is_htcstartrek() ||		// Not sure about this yet, just copying off of Elf
        machine_is_htcpharos() ||
        machine_is_artemis())
   {
#ifdef CONFIG_I2C_HTCPLD
      htcpld_chip_set(0x04, htcpld_chip_get(0x04) & 0xef);
#endif
   }

   status = 0;

   printk("HTC Bluetooth: Bluetooth turned off.\n");
}

void htc_bt_reset()
{
   htc_bt_off();
   htc_bt_on();
}

static void htc_bt_configure(int state)
{
   switch (state) {
   
   case 0:
      htc_bt_off();
      break;

   case 1:
      htc_bt_on();
      break;
   default:
      break;
   }
}


static ssize_t htc_power_show(struct device *dev, struct device_attribute *attr, char *buf)
{
   return sprintf(buf, "%d\n", status); 
}

static ssize_t htc_power_store(struct device *dev, struct device_attribute *attr, char *buf, size_t count)
{
   unsigned int power;
   power = simple_strtoul(buf, NULL, 10);
   status = power != 0 ? 1 : 0;
   htc_bt_configure(status);
   return count;
}

static DEVICE_ATTR(bt_power, 0644, htc_power_show, htc_power_store);

static ssize_t htc_pm_show(struct device *dev, struct device_attribute *attr, char *buf)
{
   return sprintf(buf, "%d\n",pm); 
}

static ssize_t htc_pm_store(struct device *dev, struct device_attribute *attr, char *buf, size_t count)
{
   unsigned int pmng;
   pmng = simple_strtoul(buf, NULL, 10);
   pm = pmng != 0 ? 1 : 0;
   return count;
}

static DEVICE_ATTR(skip_pm, 0644, htc_pm_show, htc_pm_store);

static int htc_bt_toggle(void *data, enum rfkill_state state)
{
   switch (state) {
      case RFKILL_STATE_ON:
         if (status == 0)
            htc_bt_on();
         break;

      case RFKILL_STATE_OFF:
         if (status == 1)
            htc_bt_off();
         break;
   }

   return 0;
}

static int htc_bt_probe(struct platform_device *pdev)
{
   struct rfkill *rfk;

   htc_bt_init_gpios();
   device_create_file(&pdev->dev, &dev_attr_bt_power);
   device_create_file(&pdev->dev, &dev_attr_skip_pm);

   /* Register the RFKill device */
   rfk = rfkill_allocate(&pdev->dev, RFKILL_TYPE_BLUETOOTH);

   rfk->name = "htc_bt_rfk";
   rfk->state = RFKILL_STATE_ON;
   rfk->toggle_radio = htc_bt_toggle;
   rfk->user_claim_unsupported = 1;

   /*
    * RFKill toggles devices to their ON state upon register.  For
    * us, this is bad, so we set the state to ON above to make
    * the register not actually change anything, and we then modify
    * the state back to OFF to reflect reality.
    */
   rfkill_register(rfk);
   rfk->state = RFKILL_STATE_OFF;

   /* Save away the bluetooth driver info */
   btdrv.pdev = pdev;
   btdrv.rfk = rfk;
   return 0;
}

static int htc_bt_remove(struct platform_device *pdev)
{
   htc_bt_off();
   device_remove_file(&pdev->dev, &dev_attr_bt_power);
   device_remove_file(&pdev->dev, &dev_attr_skip_pm);
	rfkill_unregister(btdrv.rfk);
   btdrv.rfk = NULL;
   return 0;
}

static int htc_bt_suspend(struct platform_device *dev, pm_message_t state)
{
   if (!pm)
      htc_bt_off();
   return 0;
}

static int htc_bt_resume(struct platform_device *dev, pm_message_t state)
{
   if (!pm)
      if (status)
         htc_bt_on();
   return 0;
}

static struct platform_driver htc_bt_driver = {
   .driver   = {
      .name     = "htc-bt",
   },
   .probe    = htc_bt_probe,
   .remove   = htc_bt_remove,
   .suspend  = htc_bt_suspend,
   .resume   = htc_bt_resume,
};

static int __init htc_bt_init(void)
{
   printk(KERN_NOTICE "HTC Bluetooth Driver\n");
   platform_driver_register( &htc_bt_driver );
   return 0;
}

static void __exit htc_bt_exit(void)
{
   platform_driver_unregister( &htc_bt_driver );
}

module_init(htc_bt_init);
module_exit(htc_bt_exit);

MODULE_AUTHOR("Crohas Fabrice");
MODULE_DESCRIPTION("HTC Bluetooth Support Driver");
MODULE_LICENSE("GPL");

