/*
 * linux/arch/arm/mach-omap1/htc-mmc.c
 *
 * Copyright (C) 2007 Instituto Nokia de Tecnologia - INdT
 * Author: Felipe Balbi <felipe.lima@indt.org.br>
 *
 * This code is based on linux/arch/arm/mach-omap2/board-n800-mmc.c, which is:
 * Copyright (C) 2006 Nokia Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <asm/arch/mmc.h>
#include <asm/arch/gpio.h>
#include <asm/mach-types.h>

#include <linux/delay.h>

#if defined(CONFIG_MMC_OMAP) || defined(CONFIG_MMC_OMAP_MODULE)

#define OMAP_MMC_REG_SYSC (0xfffb7800 + 0x32)
#define OMAP_MMC_REG_SYSS (0xfffb7800 + 0x34)

static int slot_cover_open;
static struct device *mmc_device;

static int htc_mmc_set_power(struct device *dev, int slot, int power_on,
				int vdd)
{
#ifdef CONFIG_MMC_DEBUG
	dev_dbg(dev, "Set slot %d power: %s (vdd %d)\n", slot + 1,
		power_on ? "on" : "off", vdd);
#endif
	if (slot != 0) {
		dev_err(dev, "No such slot %d\n", slot + 1);
		return -ENODEV;
	}

	return 0;
}

static int htc_mmc_set_bus_mode(struct device *dev, int slot, int bus_mode)
{
#ifdef CONFIG_MMC_DEBUG
	dev_dbg(dev, "Set slot %d bus_mode %s\n", slot + 1,
		bus_mode == MMC_BUSMODE_OPENDRAIN ? "open-drain" : "push-pull");
#endif
	if (slot != 0) {
		dev_err(dev, "No such slot %d\n", slot + 1);
		return -ENODEV;
	}

	/* Treated on upper level */

	return bus_mode;
}

static int htc_mmc_get_cover_state(struct device *dev, int slot)
{
	BUG_ON(slot != 0);

	return slot_cover_open;
}

static int htc_mmc_late_init(struct device *dev)
{
	int ret = 0;

	mmc_device = dev;

	return ret;
}

static void htc_mmc_cleanup(struct device *dev)
{
}

static struct omap_mmc_platform_data htc_mmc_data = {
	.nr_slots                       = 1,
	.switch_slot                    = NULL,
	.init                           = htc_mmc_late_init,
	.cleanup                        = htc_mmc_cleanup,
	.slots[0]       = {
		.set_power              = htc_mmc_set_power,
		.set_bus_mode           = htc_mmc_set_bus_mode,
		.get_ro                 = NULL,
		.get_cover_state        = htc_mmc_get_cover_state,
		.ocr_mask               = MMC_VDD_28_29 | MMC_VDD_30_31 |
					  MMC_VDD_32_33 | MMC_VDD_33_34,
		.name                   = "mmcblk",
	},
};

void __init htc_mmc_init(void)
{
   if (machine_is_herald()   ||
       machine_is_artemis()  ||
       machine_is_htc_opal() ||
       machine_is_htcpharos())
   {
      /* Set MUX config for SDMC, having or not mmc driver */
      omap_writel(omap_readl(OMAP850_IO_CONF_2) & ~0x0008FF00, OMAP850_IO_CONF_2);
   } else {
      /* Set MUX config for SDMC, having or not mmc driver */
      omap_writel(omap_readl(OMAP850_IO_CONF_2) & 0xFFF111FF, OMAP850_IO_CONF_2);
      /* Power on SD */
      omap_set_gpio_direction(19, 0);
      omap_set_gpio_dataout(19, 1);
   }

	/* register it */
	omap_set_mmc_info(1, &htc_mmc_data);
}
#endif
