/*
 * File: drivers/video/omap/lcd-prophet.c
 *
 * LCD panel support for the HTC Prophet
 *
 * Copyright (C) linwizard.sourceforge.net
 * Author: Angelo Arrifano <miknix@gmail.com>
 * Based on lcd_h4 by Imre Deak <imre.deak@nokia.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <linux/module.h>
#include <linux/platform_device.h>

#include <asm/arch/omapfb.h>

#define OMAP_LCDC_SUBPANEL 0xfffec014 

static int htcprophet_panel_init(struct lcd_panel *panel,
                                struct omapfb_device *fbdev)
{
	return 0;
}

static void htcprophet_panel_cleanup(struct lcd_panel *panel)
{
}

static int htcprophet_panel_enable(struct lcd_panel *panel)
{

	return 0;
}

static void htcprophet_panel_disable(struct lcd_panel *panel)
{
}

static unsigned long htcprophet_panel_get_caps(struct lcd_panel *panel)
{
	return 0;
}

/* Not sure */
struct lcd_panel htcprophet_panel1 = {
	.name		= "lcd1",
	.config		= OMAP_LCDC_PANEL_TFT |
	              OMAP_LCDC_INV_VSYNC |
	              OMAP_LCDC_INV_HSYNC |
	              OMAP_LCDC_HSVS_OPPOSITE,

	.bpp		= 16,
	.data_lines	= 16,
	.x_res		= 240,
	.y_res		= 320,
	.pixel_clock	= 5500,
	.hsw		= 10,
	.hfp		= 10,
	.hbp		= 20,
	.vsw		= 2,
	.vfp		= 2,
	.vbp		= 3,

	.init		= htcprophet_panel_init,
	.cleanup	= htcprophet_panel_cleanup,
	.enable		= htcprophet_panel_enable,
	.disable	= htcprophet_panel_disable,
	.get_caps	= htcprophet_panel_get_caps,
};

/* Found on HTC Prophets */
struct lcd_panel htcprophet_panel2 = {
	.name		= "lcd2",
	.config		= OMAP_LCDC_PANEL_TFT |
	              OMAP_LCDC_INV_HSYNC |
					  OMAP_LCDC_INV_PIX_CLOCK,
	.bpp		= 16,
	.data_lines	= 16,
	.x_res		= 240,
	.y_res		= 320,
	.pixel_clock = 6093,
	.hsw		= 10,
	.hfp		= 10,
	.hbp		= 20,
	.vsw		= 3,
	.vfp		= 2,
	.vbp		= 2,

	.init		= htcprophet_panel_init,
	.cleanup	= htcprophet_panel_cleanup,
	.enable		= htcprophet_panel_enable,
	.disable	= htcprophet_panel_disable,
	.get_caps	= htcprophet_panel_get_caps,
};

/* Found on HTC Elfs */
struct lcd_panel htcprophet_panel3 = {
	.name		= "lcd3",
	.config		= OMAP_LCDC_PANEL_TFT |
	              OMAP_LCDC_INV_HSYNC |
					  OMAP_LCDC_INV_PIX_CLOCK,
	.bpp		= 16,
	.data_lines	= 16,
	.x_res		= 240,
	.y_res		= 320,
	.pcd = 18,
	.hsw		= 4,
	.hfp		= 8,
	.hbp		= 4,
	.vsw		= 3,
	.vfp		= 8,
	.vbp		= 5,

	.init		= htcprophet_panel_init,
	.cleanup	= htcprophet_panel_cleanup,
	.enable		= htcprophet_panel_enable,
	.disable	= htcprophet_panel_disable,
	.get_caps	= htcprophet_panel_get_caps,
};


static int htcprophet_panel_probe(struct platform_device *pdev)
{
	u32 type;
	
	type = omap_readl(OMAP_LCDC_SUBPANEL);
	switch(type){
	case 0x5c00f000:
		printk("LCD: Panel type 2\n");
		omapfb_register_panel(&htcprophet_panel2);
		break;
	case 0x5c000000:
		printk("LCD: Panel type 1\n");
		omapfb_register_panel(&htcprophet_panel3);
		break;
	default:
		printk("LCD: Unknown Panel type\n");
		omapfb_register_panel(&htcprophet_panel1);
	}

	return 0;
}

static int htcprophet_panel_remove(struct platform_device *pdev)
{
	return 0;
}

static int htcprophet_panel_suspend(struct platform_device *pdev, pm_message_t mesg)
{
	return 0;
}

static int htcprophet_panel_resume(struct platform_device *pdev)
{
	return 0;
}

struct platform_driver htcprophet_panel_driver = {
	.probe		= htcprophet_panel_probe,
	.remove		= htcprophet_panel_remove,
	.suspend	= htcprophet_panel_suspend,
	.resume		= htcprophet_panel_resume,
	.driver		= {
		.name	= "lcd_htcprophet",
		.owner	= THIS_MODULE,
	},
};

static int htcprophet_panel_drv_init(void)
{
	return platform_driver_register(&htcprophet_panel_driver);
}

static void htcprophet_panel_drv_cleanup(void)
{
	platform_driver_unregister(&htcprophet_panel_driver);
}

module_init(htcprophet_panel_drv_init);
module_exit(htcprophet_panel_drv_cleanup);

